/* Copyright 2004, Rajiv Shivane. All Rights Reserved. */

/*
 * 
 * Jaskell specific functions used in chroma coding
 * <pre/> elements with class "jsccJaskell"
 * 
 */

supportedLanguages["jsccJaskell"] = new Object();
supportedLanguages["jsccJaskell"].main = jsccJaskell;

var keyWordsRE = createWordSearchRegEx(new Array("if", "then", "else", "where", "end", "let", "this", "true", "false", "and", "or", "not"));
var primitiveRE = createWordSearchRegEx(new Array("jaskell", "prelude", "const", "id", "flip", "map", "fold", 
  "switch", "assert", "case", "default", "auto", "try", "catch", "finally", "throw", 
  "new", "import", "null", "implements", "extends", "includes", "do", "instanceof",
  "is", "overload", "now", "classloader", "class","array","error"));

function markKeywords(text) {
    var text;

		text = text.replace(primitiveRE, "$1<span class='jsccPrimitive'>$2</span>$3");
    text = text.replace(keyWordsRE, "$1<span class='jsccKeyword'>$2</span>$3");

    return text;
}

function matchParens(text) {
    var text;
    var parens = new RegExp ("(\{|\})","mg");
    return text.replace(parens, 
                function (str, p1, p2, offset, s) {
                    if(str=="{")
                        return"<span class='jsccBlockOpen'>{</span><span class='jsccBlockBody'>";
                    else
                        return"</span><span class='jsccBlockClose'>}</span>";
                });
}


function jsccJaskell(curElement){
    var curElement;
    var origText = curElement.innerHTML;
    var ccText = "";
    var lastOffset = 0;

    //Single line comment
    var slcREStr = "//.*"; 
    //String literal
    var stringREStr ="\"(?:(?:\\\\.)|(?:[^\"]))*\"";
    //Character literal
    var charREStr ="'(?:(?:\\\\.)|(?:[^'\r\n]))'"; //TODO: unicode characters need to be taken care of
    //Multiline comment
    var mlcREStr = "(/\\*(?:.|[\\n\\r])*?\\*/)|(\\[\\*\\*(?:.|[\\n\\r])*?\\*\\*\\])";

    //var slcRE = new RegExp ("("+slcREStr+")","img");
    var stringRE = new RegExp ("("+stringREStr+")","img");
    var charRE = new RegExp ("("+charREStr+")","img");
    //var mlcRE = new RegExp ("("+mlcREStr+")","img");

    //(single-line-comment|string-literal|multi-line-comment)
    var myRE = new RegExp ("((?:"+slcREStr+")|(:?"+stringREStr+")|(?:"+mlcREStr+")|(?:"+charREStr+"))","img");
    var origLen = origText.length;
    origText.replace(myRE, 
            function (str, p1, p2, p3, p4, offset, s) {
                var tmp=s.substring(lastOffset, offset);
                tmp=makeHtmlSafe(tmp);
                tmp=markKeywords(tmp);
                tmp=matchParens(tmp);
                //ccText+=tmp;
                if((str.search(stringRE)==0) || (str.search(charRE)==0))
                    ccText+=tmp+"<span class='jsccLiteral'>"+makeHtmlSafe(str)+"</span>";
                else
                    ccText+=tmp+"<span class='jsccComment'>"+makeHtmlSafe(str)+"</span>";
                lastOffset=offset+str.length;
                return str;
            });
    if(lastOffset<origLen){
        var tmp=origText.substring(lastOffset, origText.length);
        tmp=makeHtmlSafe(tmp);
        tmp=markKeywords(tmp);
        tmp=matchParens(tmp);
        ccText+=tmp;
    }
    curElement.innerHTML = ccText;
    var spans = curElement.getElementsByTagName("span");
    registerBlockClickListeners(spans);     
}
